from typing import Dict, List, Optional, Union

import infima_client.api.prediction_service.prediction_service_get_cohort as prediction_service_get_cohort
import infima_client.api.prediction_service.prediction_service_get_pools as prediction_service_get_pools
import infima_client.api.prediction_service.prediction_service_get_portfolio as prediction_service_get_portfolio
from infima_client.models import *

from .client import Client
from .types import UNSET, Response, Unset


class prediction:
    def __init__(self, client: Client):
        self.client = client
        self._detailed = predictionDetailed(client)

    def get_cohort(
        self,
        cohort: str,
        as_ofyear: Union[Unset, int] = UNSET,
        as_ofmonth: Union[Unset, int] = UNSET,
        as_ofday: Union[Unset, int] = UNSET,
        convention: Union[
            Unset, PredictionService_GetCohortConvention
        ] = PredictionService_GetCohortConvention.PERIOD,
        scenario: Union[
            Unset, PredictionService_GetCohortScenario
        ] = PredictionService_GetCohortScenario.BASELINE,
    ) -> Optional[
        Union[
            CohortPrediction,
            CohortPredictionBadRequest,
            CohortPredictionUnauthorized,
            CohortPredictionNotFound,
            CohortPredictionError,
        ]
    ]:

        """
        Fetch prepayment prediction for a single cohort for an as-of date, default to today.

        Args:
            cohort: Cohort name to fetch aggregated prepayment prediction.
            as_ofyear: Year of the date. Must be from 1 to 9999, or 0 to specify a date without a year
            as_ofmonth: Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
            as_ofday: Day of a month. Must be from 1 to 31 and valid for the year and month, or 0 to specify a year by itself or a year and month where the day isn't significant.Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
            convention: Convention to use for the date, can be PERIOD or FACTOR_DATE.
            scenario: None

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return prediction_service_get_cohort.sync(
            client=self.client,
            cohort=cohort,
            as_ofyear=as_ofyear,
            as_ofmonth=as_ofmonth,
            as_ofday=as_ofday,
            convention=convention,
            scenario=scenario,
        )

    def get_pools(
        self,
        cusips: List[str],
        as_ofyear: Union[Unset, int] = UNSET,
        as_ofmonth: Union[Unset, int] = UNSET,
        as_ofday: Union[Unset, int] = UNSET,
        convention: Union[
            Unset, PredictionService_GetPoolsConvention
        ] = PredictionService_GetPoolsConvention.PERIOD,
        scenario: Union[
            Unset, PredictionService_GetPoolsScenario
        ] = PredictionService_GetPoolsScenario.BASELINE,
    ) -> Optional[
        Union[
            PoolPredictions,
            PoolPredictionsBadRequest,
            PoolPredictionsUnauthorized,
            PoolPredictionsNotFound,
            PoolPredictionsError,
        ]
    ]:

        """
        Fetch individual pool prepayment predictions for an as-of date, default to today.

        Args:
            cusips: Pool CUSIP or CUSIPS to fetch individual prepayment predictions.
            as_ofyear: Year of the date. Must be from 1 to 9999, or 0 to specify a date without a year
            as_ofmonth: Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
            as_ofday: Day of a month. Must be from 1 to 31 and valid for the year and month, or 0 to specify a year by itself or a year and month where the day isn't significant.Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
            convention: Convention to use for the date, can be PERIOD or FACTOR_DATE.
            scenario: None

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return prediction_service_get_pools.sync(
            client=self.client,
            cusips=cusips,
            as_ofyear=as_ofyear,
            as_ofmonth=as_ofmonth,
            as_ofday=as_ofday,
            convention=convention,
            scenario=scenario,
        )

    def get_portfolio(
        self,
        cusips: List[str],
        as_ofyear: Union[Unset, int] = UNSET,
        as_ofmonth: Union[Unset, int] = UNSET,
        as_ofday: Union[Unset, int] = UNSET,
        convention: Union[
            Unset, PredictionService_GetPortfolioConvention
        ] = PredictionService_GetPortfolioConvention.PERIOD,
        scenario: Union[
            Unset, PredictionService_GetPortfolioScenario
        ] = PredictionService_GetPortfolioScenario.BASELINE,
    ) -> Optional[
        Union[
            PortfolioPrediction,
            PortfolioPredictionBadRequest,
            PortfolioPredictionUnauthorized,
            PortfolioPredictionNotFound,
            PortfolioPredictionError,
        ]
    ]:

        """
        Weighted aggregate of the prepayment prediction of the portfolio components for an as-of date, default to today.

        Args:
            cusips: Pool CUSIPS to fetch aggregated prepayment prediction.
            as_ofyear: Year of the date. Must be from 1 to 9999, or 0 to specify a date without a year
            as_ofmonth: Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
            as_ofday: Day of a month. Must be from 1 to 31 and valid for the year and month, or 0 to specify a year by itself or a year and month where the day isn't significant.Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
            convention: Convention to use for the date, can be PERIOD or FACTOR_DATE.
            scenario: None

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return prediction_service_get_portfolio.sync(
            client=self.client,
            cusips=cusips,
            as_ofyear=as_ofyear,
            as_ofmonth=as_ofmonth,
            as_ofday=as_ofday,
            convention=convention,
            scenario=scenario,
        )


class predictionDetailed:
    def __init__(self, client: Client):
        self.client = client

    def get_cohort(
        self,
        cohort: str,
        as_ofyear: Union[Unset, int] = UNSET,
        as_ofmonth: Union[Unset, int] = UNSET,
        as_ofday: Union[Unset, int] = UNSET,
        convention: Union[
            Unset, PredictionService_GetCohortConvention
        ] = PredictionService_GetCohortConvention.PERIOD,
        scenario: Union[
            Unset, PredictionService_GetCohortScenario
        ] = PredictionService_GetCohortScenario.BASELINE,
    ) -> Response[
        Union[
            CohortPrediction,
            CohortPredictionBadRequest,
            CohortPredictionUnauthorized,
            CohortPredictionNotFound,
            CohortPredictionError,
        ]
    ]:

        return prediction_service_get_cohort.sync_detailed(
            client=self.client,
            cohort=cohort,
            as_ofyear=as_ofyear,
            as_ofmonth=as_ofmonth,
            as_ofday=as_ofday,
            convention=convention,
            scenario=scenario,
        )

    def get_pools(
        self,
        cusips: List[str],
        as_ofyear: Union[Unset, int] = UNSET,
        as_ofmonth: Union[Unset, int] = UNSET,
        as_ofday: Union[Unset, int] = UNSET,
        convention: Union[
            Unset, PredictionService_GetPoolsConvention
        ] = PredictionService_GetPoolsConvention.PERIOD,
        scenario: Union[
            Unset, PredictionService_GetPoolsScenario
        ] = PredictionService_GetPoolsScenario.BASELINE,
    ) -> Response[
        Union[
            PoolPredictions,
            PoolPredictionsBadRequest,
            PoolPredictionsUnauthorized,
            PoolPredictionsNotFound,
            PoolPredictionsError,
        ]
    ]:

        return prediction_service_get_pools.sync_detailed(
            client=self.client,
            cusips=cusips,
            as_ofyear=as_ofyear,
            as_ofmonth=as_ofmonth,
            as_ofday=as_ofday,
            convention=convention,
            scenario=scenario,
        )

    def get_portfolio(
        self,
        cusips: List[str],
        as_ofyear: Union[Unset, int] = UNSET,
        as_ofmonth: Union[Unset, int] = UNSET,
        as_ofday: Union[Unset, int] = UNSET,
        convention: Union[
            Unset, PredictionService_GetPortfolioConvention
        ] = PredictionService_GetPortfolioConvention.PERIOD,
        scenario: Union[
            Unset, PredictionService_GetPortfolioScenario
        ] = PredictionService_GetPortfolioScenario.BASELINE,
    ) -> Response[
        Union[
            PortfolioPrediction,
            PortfolioPredictionBadRequest,
            PortfolioPredictionUnauthorized,
            PortfolioPredictionNotFound,
            PortfolioPredictionError,
        ]
    ]:

        return prediction_service_get_portfolio.sync_detailed(
            client=self.client,
            cusips=cusips,
            as_ofyear=as_ofyear,
            as_ofmonth=as_ofmonth,
            as_ofday=as_ofday,
            convention=convention,
            scenario=scenario,
        )
